import '../../styles';
import * as React from 'react';
import { Controlled as CodeMirrorControlled } from 'react-codemirror2';
import { Subject } from 'rxjs';
import { share, takeUntil } from 'rxjs/operators';
import { css, is, time } from '../../common';
if (is.browser) {
    require('codemirror/mode/mathematica/mathematica.js');
    require('codemirror/mode/javascript/javascript.js');
}
export class FormulaInput extends React.PureComponent {
    constructor() {
        super(...arguments);
        this.state = { isLoaded: false };
        this.unmounted$ = new Subject();
        this.state$ = new Subject();
        this.editorRef = (ref) => (this.editor = ref && ref.editor);
        this._events$ = new Subject();
        this.events$ = this._events$.pipe(takeUntil(this.unmounted$), share());
        this.fire = (e) => this._events$.next(e);
        this.focusHandler = (isFocused) => {
            return () => {
                if (isFocused) {
                    this.fire({ type: 'INPUT/formula/focus', payload: {} });
                }
                if (!isFocused) {
                    this.fire({ type: 'INPUT/formula/blur', payload: {} });
                }
            };
        };
        this.handleBeforeChange = (editor, data, value) => {
            const { isMultiLine = false, maxLength } = this.props;
            const change = data;
            const char = change.text[0];
            if (char.includes('\t')) {
                let isCancelled = false;
                this.fire({
                    type: 'INPUT/formula/tab',
                    payload: {
                        cancel: () => (isCancelled = true),
                        get isCancelled() {
                            return isCancelled;
                        },
                    },
                });
                if (isCancelled) {
                    return;
                }
            }
            if (!isMultiLine) {
                const wasCancelled = singleLineOnly(change);
                if (wasCancelled) {
                    return;
                }
            }
            if (maxLength !== undefined) {
                const clippedValue = formatValue(value, { maxLength, isMultiLine });
                if (clippedValue !== value) {
                    if (change.update && change.origin === '+input') {
                        change.update(undefined, undefined, [clippedValue]);
                        value = clippedValue;
                    }
                }
            }
            const from = this.props.value || '';
            const to = value;
            const isMax = maxLength === undefined ? null : to.length === maxLength;
            this.fire({ type: 'INPUT/formula/change', payload: { from, to, isMax, char } });
        };
    }
    componentWillMount() {
        const { events$ } = this.props;
        this.state$.pipe(takeUntil(this.unmounted$)).subscribe(e => this.setState(e));
        if (events$) {
            this.events$.subscribe(e => events$.next(e));
        }
    }
    componentDidMount() {
        const { focusOnLoad, selectOnLoad } = this.props;
        time.delay(0, () => {
            this.setState({ isLoaded: true }, () => {
                if (focusOnLoad) {
                    this.focus();
                }
                if (selectOnLoad) {
                    this.selectAll();
                }
            });
        });
    }
    componentWillUnmount() {
        this.unmounted$.next();
        this.unmounted$.complete();
    }
    get isFocused() {
        return this.editor ? this.editor.hasFocus() : false;
    }
    get height() {
        const { isMultiLine = false, height } = this.props;
        if (height !== undefined) {
            return height;
        }
        if (!isMultiLine && height === undefined) {
            return 21;
        }
        return;
    }
    focus() {
        if (this.editor) {
            this.editor.focus();
        }
    }
    redraw() {
        this.forceUpdate();
        if (this.editor) {
            this.editor.refresh();
        }
    }
    selectAll() {
        this.editor.execCommand('selectAll');
    }
    render() {
        const { isMultiLine = false, maxLength, mode = 'mathematica' } = this.props;
        const height = this.height;
        const value = formatValue(this.props.value, { maxLength, isMultiLine });
        const styles = {
            base: css({
                height,
                Scroll: isMultiLine,
                overflow: 'hidden',
                lineHeight: 1,
            }),
        };
        const el = this.state.isLoaded && (React.createElement(CodeMirrorControlled, { ref: this.editorRef, options: { mode }, value: value, onBeforeChange: this.handleBeforeChange, onFocus: this.focusHandler(true), onBlur: this.focusHandler(false) }));
        return (React.createElement("div", Object.assign({ className: 'FormulaInput' }, css(styles.base, this.props.style)), el));
    }
}
function singleLineOnly(change) {
    const typedNewLine = change.origin === '+input' && typeof change.text === 'object' && change.text.join('') === '';
    if (typedNewLine) {
        return change.cancel();
    }
    const pastedNewLine = change.origin === 'paste' && typeof change.text === 'object' && change.text.length > 1;
    if (pastedNewLine && change.update) {
        const newText = change.text.join(' ');
        return change.update(undefined, undefined, [newText]);
    }
    return null;
}
function formatValue(value, options) {
    const { maxLength, isMultiLine = false } = options;
    value = value || '';
    if (!isMultiLine) {
        value = value.replace(/\n/g, '');
    }
    if (maxLength !== undefined && value.length > maxLength) {
        value = value.substr(0, maxLength);
    }
    return value;
}
